@@ -1,4 +1,4 @@
-/* $XdotOrg: xc/programs/xdm/xdmcp.c,v 1.1.4.5.2.1 2004/03/04 17:48:56 eich Exp $ */
+/* $XdotOrg: xc/programs/xdm/xdmcp.c,v 1.4 2004/08/07 19:22:01 alanc Exp $ */
 /* $Xorg: xdmcp.c,v 1.4 2001/02/09 02:05:41 xorgcvs Exp $ */
 /*
 
@@ -943,13 +943,20 @@ request_respond (
     ARRAY8Ptr	    connectionAddress;
 
     Debug ("Request respond %d\n", length);
+    connectionTypes.length = 0;
     connectionTypes.data = 0;
+    connectionAddresses.length = 0;
     connectionAddresses.data = 0;
+    authenticationName.length = 0;
     authenticationName.data = 0;
+    authenticationData.length = 0;
     authenticationData.data = 0;
+    authorizationNames.length = 0;
     authorizationNames.data = 0;
     authorizationName.length = 0;
+    authorizationName.data = 0;
     authorizationData.length = 0;
+    authorizationData.data = 0;
     manufacturerDisplayID.data = 0;
     if (XdmcpReadCARD16 (&buffer, &displayNumber) &&
 	XdmcpReadARRAY16 (&buffer, &connectionTypes) &&
@@ -1190,7 +1197,8 @@ manage (
 					 &pdpy->connectionAddress,
 					 from,
 					 pdpy->displayNumber);
-	    Debug ("Computed display name: %s\n", name);
+	    Debug ("Computed display name: %s for: %s\n",
+		   name, (char *)pdpy->connectionAddress.data);
 	    if (!name)
 	    {
 		send_failed (from, fromlen, "(no name)", sessionID, 
